pro corr_jumps_pacs, njumps, file_scalars=file_scalars, ind_scans=ind_scans, ind_subscans=ind_subscans, $
                     maxnoise=maxnoise, f_fwhm=f_fwhm, time_1=time_1, t_proc=t_proc


;;; detects and masks brightness discontinuities affecting whole array rows (PACS),
;;; using the current map as a reference

;;;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;;; copyright 2007-2011 H\'el\`ene Roussel
;;; roussel@iap.fr
;;; Institut d'Astrophysique de Paris - Universit\'e Pierre et Marie Curie, France

;;; download and documentation: http://www2.iap.fr/users/roussel/herschel/

;;; Scanamorphos is distributed under the terms of the GNU General Public License
;;; (http://www.gnu.org/licenses/gpl.html).

;;; introduced December 2010: version 7

;;;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



crs = string(13b) + string(10b)
restore, file_scalars

restore, file_pixcoords
restore, file_signal
restore, file_times


;;; simplified reference map on a fine grid:

mult_xy = round(4. * f_fwhm / fact)
dimx_fine = dimx * mult_xy
dimy_fine = dimy * mult_xy
map_ref = fltarr(dimx_fine, dimy_fine)
indxy_beam = floor(mult_xy * x_bolos) + dimx_fine * floor(mult_xy * y_bolos)
wnan = where(finite(signal) lt 1 or x_bolos lt 0. or x_bolos ge dimx $
   or y_bolos lt 0. or y_bolos ge dimy, cw_nan)
if cw_nan ge 1 then indxy_beam(wnan) = -9L
wnan = 0
histo_indxy = histogram(indxy_beam, reverse_indices=revind_fine, min=0., max=dimx_fine*dimy_fine-1)
indxy_beam = 0
wproj_fine = where(histo_indxy ge 1L, nproj_fine)
histo_indxy = 0

for k = 0L, nproj_fine - 1 do begin
   revind_0 = revind_fine(wproj_fine(k))
   revind_1 = revind_fine(wproj_fine(k)+1)
   cp = revind_1 - revind_0
   wp = revind_fine(revind_0:revind_1-1)
   map_ref(wproj_fine(k)) = median(signal(wp), /even)
endfor



off = 2
naver = ceil(3. / timestep)
nmask = ceil(3. / timestep)
nmask_max = 100L * nmask
njumps = 0L


nb_line = 16L
nlines = nb / nb_line
for kl = 0L, nlines - 1L do begin
   wb = nb_line * kl + lindgen(nb_line)
   w = where(finite(signal(*, wb)) lt 1, cw)
   mask_bolo = 1 + lonarr(nt, nb_line)
   if cw ge 1 then mask_bolo(w) = 0L
   norm = total(mask_bolo, 2)
   norm_denom = norm
   w = where(norm eq 0., cw)
   if cw ge 1 then norm_denom(w) = 1.
   aver_line = total(signal(*, wb) * mask_bolo, 2, /nan) / norm_denom
   if cw ge 1 then aver_line(w) = !values.f_nan

   indxy_ref = floor(mult_xy * x_bolos(*, wb)) + dimx_fine * floor(mult_xy * y_bolos(*, wb))
   noise_level = 1.5 * median(maxnoise(*, wb), /even)
   noise_resi = noise_level / 3.


   for subs = 0L, nsubscans - 1L do begin
      wdef = where(finite(aver_line(ind_subscans(subs):ind_subscans(subs+1)-1)) gt 0, cdef)
      if cdef ge 4 * naver then begin
         aver_scan = aver_line(ind_subscans(subs)+wdef)
         ref = total(map_ref(indxy_ref(ind_subscans(subs)+wdef, *)) * mask_bolo(ind_subscans(subs)+wdef, *), 2) $
            / total(mask_bolo(ind_subscans(subs)+wdef, *), 2)
         ref -= median(ref)
         aver_scan -= ref
         diff = abs(aver_scan(off:cdef-1) - aver_scan(0:cdef-1-off))
         diff(0:2*naver+off) = 0.
         diff(cdef-1-off-2*naver-off:cdef-1-off) = 0.
         diff_time = time_s(ind_subscans(subs)+wdef(off:cdef-1)) - time_s(ind_subscans(subs)+wdef(0:cdef-1-off))
         w = where(diff_time gt 10. * maxtimediff, cw)
         if cw ge 1 then diff(w) = 0.
         wj = where(diff gt noise_level, cj)
         while cj ge 1 do begin
            kj = wj(0)
            tab0 = aver_scan(kj-2*naver:kj-naver)
            tab1 = aver_scan(kj-naver+1:kj)
            tab2 = aver_scan(kj+off:kj+off+naver-1)
            tab3 = aver_scan(kj+off+naver:kj+off+2*naver-1)
            b0 = median(tab0, /even)
            b1 = median(tab1, /even)
            b2 = median(tab2, /even)
            b3 = median(tab3, /even)
            delta = min([abs(b2 - b1), abs(b2 - b0)])
            sig1 = sqrt(mean((tab1 - b1)^2.))
            sig2 = sqrt(mean((tab2 - b2)^2.))

            if (delta gt max([noise_level, 3. * sig1, 3. * sig2, 2. * abs(b3 - b2), 2. * abs(b1 - b0)]) $
               and sig1 lt median(maxnoise) and flag_nza(ind_subscans(subs)+wdef(kj)) gt 0) then begin
               nmask_tot = 0L
               delta_resi = 2. * noise_resi
               while(delta_resi gt noise_resi and kj+nmask_tot+nmask lt cdef - 1 and nmask_tot lt nmask_max) do begin
                  nmask_tot += nmask
                  tab4 = aver_scan(kj+nmask_tot:min([kj+nmask_tot+nmask, cdef-1]))
                  b4 = median(tab4, /even)
                  delta_resi = min([abs(b4 - b0), abs(b4 - b1)])
               endwhile
               if (delta_resi gt noise_resi and nmask_tot lt nmask_max) then nmask_tot += nmask
               nmask_tot += (nmask / 2)

               ;;; beta-release module: interactive mode enforced to check detections
               time_2 = systime(/seconds)
               t_proc += (time_2 - time_1)
               wset, 0
               ind1 = max([0, kj - 1500])
               ind2 = min([cdef - 1, max([kj + 1500, kj+nmask_tot+500])])
               otherplot, time_s(ind_subscans(subs)+wdef(ind1:ind2)), aver_scan(ind1:ind2), $
                  xtitle='time (s)', ytitle='brightness (Jy/bol.)', $
                  title='average of array row ' + chain(kl) + ' in subscan ' + chain(subs) + ' (masked values in red)'
               otherplot, /over, color=200, time_s, flag_nza * max(aver_scan(ind1:ind2))
               otherplot, /over, color=10, time_s(ind_subscans(subs)+wdef(kj:min([kj+nmask_tot, cdef-1]))), $
                  aver_scan(kj:min([kj+nmask_tot, cdef-1]))
               res = dialog_message('Confirm and mask discontinuity ?', /center, /question)
               time_1 = systime(/seconds)

               if res eq 'Yes' then begin
                  diff(kj:min([kj+nmask_tot, cdef-1-off])) = 0.
                  signal(ind_subscans(subs)+wdef(kj):ind_subscans(subs)+wdef(min([kj+nmask_tot, cdef-1])), wb) = !values.f_nan
                  njumps += 1
               endif
            endif
            diff(kj:kj+off) = 0.
            wj = where(diff gt noise_level, cj)
         endwhile
      endif
   endfor
endfor

if njumps gt 0 then begin
   print, crs + 'masked ' + chain(njumps) + ' brightness jump(s) affecting whole array row(s)'
   save, file=file_signal, signal, /xdr
   prepare_proj_fine, file_scalars=file_scalars, ind_scans=ind_scans
   prepare_proj_coarse, file_scalars=file_scalars
endif

end
